import {ResponseFactory} from './ResponseFactory'
import {Request} from '../lifecycle/Request'

/**
 * Helper function that creates a new HTMLResponseFactory.
 * @param value
 */
export function html(value: string): HTMLResponseFactory {
    return new HTMLResponseFactory(value)
}

/**
 * Response factory that writes a string to the response as HTML.
 */
export class HTMLResponseFactory extends ResponseFactory {
    constructor(
        public readonly value: string,
    ) {
        super()
    }

    public async write(request: Request): Promise<Request> {
        request = await super.write(request)
        request.response.setHeader('Content-Type', 'text/html; charset=utf-8')
        request.response.body = this.value
        return request
    }
}
