import {ErrorWithContext} from '../../util'
import {Container} from '../../di'
import {ResultIterable} from './result/ResultIterable'
import {QueryRow} from '../types'
import {AbstractBuilder} from './AbstractBuilder'
import {AbstractResultIterable} from './result/AbstractResultIterable'

/**
 * Implementation of the abstract builder class that returns simple QueryRow objects.
 */
export class Builder extends AbstractBuilder<QueryRow> {
    public getNewInstance(): AbstractBuilder<QueryRow> {
        return Container.getContainer().make<Builder>(Builder)
    }

    public getResultIterable(): AbstractResultIterable<QueryRow> {
        if ( !this.registeredConnection ) {
            throw new ErrorWithContext(`No connection specified to fetch iterator for query.`)
        }

        return Container.getContainer().make<ResultIterable>(ResultIterable, this, this.registeredConnection)
    }
}
