import {ViewEngine} from './ViewEngine'
import {Injectable} from '../di'
import * as pug from 'pug'

/**
 * Implementation of the ViewEngine class that renders Pug/Jade templates.
 */
@Injectable()
export class PugViewEngine extends ViewEngine {
    /** A cache of compiled templates. */
    protected compileCache: {[key: string]: ((locals?: pug.LocalsObject) => string)} = {}

    public renderString(templateString: string, locals: { [p: string]: any }): string | Promise<string> {
        return pug.compile(templateString, this.getOptions())(locals)
    }

    public renderByName(templateName: string, locals: { [p: string]: any }): string | Promise<string> {
        let compiled = this.compileCache[templateName]
        if ( compiled ) {
            return compiled(locals)
        }

        if ( !templateName.endsWith('.pug') ) {
            templateName += '.pug'
        }
        const filePath = this.path.concat(...templateName.split(':'))
        compiled = pug.compileFile(filePath.toLocal, this.getOptions())

        this.compileCache[templateName] = compiled
        return compiled(locals)
    }

    /**
     * Get the object of options passed to Pug's compile methods.
     * @protected
     */
    protected getOptions(): pug.Options {
        return {
            basedir: this.path.toLocal,
            debug: this.debug,
            compileDebug: this.debug,
            globals: [],
        }
    }
}
